//*****************************************************************************
//* Library of different utilities. Routines file.
//*    Some routines were taken from "Numerical Recipes" by Press, 
//*                                              Teukolsky, Vetterling, Flanery
//*    Realized by P.V.Nazarov, 2002
//*****************************************************************************
///////////////////////////////////////////////////////////////////////////////
#pragma hdrstop
#include "LibUtil.h"
//=============================================================================
void LibError(char error_text[],char error_proc[],int exit_code)
{
  fprintf(stderr,"\nRUN-TIME ERROR in '%s': ",error_proc);
  fprintf(stderr,"...%s...\n",error_text);
  if (exit_code) {
    fprintf(stderr,"...now exiting to system with the exit_code=%d...\n",exit_code);
    getch();
    exit(exit_code);
  }
}
//-----------------------------------------------------------------------------
void LibError(char error_text[],int exit_code)
{
  fprintf(stderr,"\nRUN-TIME ERROR: ");
  fprintf(stderr,"...%s...\n",error_text);
  if (exit_code) {
    fprintf(stderr,"...now exiting to system with the exit_code=%d...\n",exit_code);
    getch();
    exit(exit_code);
  }
}
//------------------------------------------------------------------------------
void LibError(char error_text[])
{
  fprintf(stderr,"\nRUN-TIME ERROR: ");
  fprintf(stderr,"...%s...\n",error_text);
  getch();
  exit(1);
}
//=============================================================================
/*double pow(double x, double n)
{
 if (fabs(n)<1e-20) return 1;
 else return exp(n*log(x));
}*/

double pow(double x, int n)
{
 if (n==0) return 1;
 else return x*pow(x,n-1);
}

//-----------------------------------------------------------------------------
int round(double r)
{
  if (r-(int)r>=0.5)
    return (int)r+1;
  else
    return (int)r;
}
int Round(double r)
{
  if (r-(int)r>=0.5)
    return (int)r+1;
  else
    return (int)r;
}
//------------------------------------------------------------------------------
char UpCase(char c)
{
  if ((c>96)&&(c<123)) return c-32;
  return c;
}
char upcase(char c)
{
  if ((c>96)&&(c<123)) return c-32;
  return c;
}
//-----------------------------------------------------------------------------
char DownCase(char c)
{
  if ((c>64)&&(c<91)) return c+32;
  return c;
}
char downcase(char c)
{
  if ((c>64)&&(c<91)) return c+32;
  return c;
}
//////////////////////////////// VECTORS //////////////////////////////////////
//=============================================================================
float *NewVectorF(long nl, long nh) {return NewVector(FLOAT_VAR,nl,nh);}
float *NewVectorF(long n)           {return NewVector(FLOAT_VAR,0,n-1);}
float *NewVector(float, long n)     {return NewVector(FLOAT_VAR,0,n-1);}
float *NewVector(float, long nl, long nh)
{
  float *v;
  v=(float *)malloc((size_t) ((nh-nl+1+LIB_VECTOR_END)*sizeof(float)));
  if (!v) LibError("allocation failure in Vectorf()");
  v+=-nl+LIB_VECTOR_END;
  for (long i=nl;i<=nh;i++) v[i]=FLOAT_VAR;
  return v;
}
//-----------------------------------------------------------------------------
double *NewVectorD(long nl, long nh) {return NewVector(DOUBLE_VAR,nl,nh);}
double *NewVectorD(long n)           {return NewVector(DOUBLE_VAR,0,n-1);}
double *NewVector(double, long n)    {return NewVector(DOUBLE_VAR,0,n-1);}
double *NewVector(double, long nl, long nh)
{
  double *v;
  v=(double *)malloc((size_t) ((nh-nl+1+LIB_VECTOR_END)*sizeof(double)));
  if (!v) LibError("allocation failure in Vectord()");
  v+=-nl+LIB_VECTOR_END;
  for (long i=nl;i<=nh;i++) v[i]=DOUBLE_VAR;
  return v;
}
//-----------------------------------------------------------------------------
int *NewVectorI(long nl, long nh) {return NewVector(INT_VAR,nl,nh);}
int *NewVectorI(long n)           {return NewVector(INT_VAR,0,n-1);}
int *NewVector(int, long n)       {return NewVector(INT_VAR,0,n-1);}
int *NewVector(int, long nl, long nh)
{
  int *v;
  v=(int *)malloc((size_t) ((nh-nl+1+LIB_VECTOR_END)*sizeof(int)));
  if (!v) LibError("allocation failure in Vectori()");
  v+=-nl+LIB_VECTOR_END;
  for (long i=nl;i<=nh;i++) v[i]=INT_VAR;
  return v;
}
//-----------------------------------------------------------------------------
char *NewVectorC(long nl, long nh) {return NewVector(CHAR_VAR,nl,nh);}
char *NewVectorC(long n)           {return NewVector(CHAR_VAR,0,n-1);}
char *NewVector(char, long n)      {return NewVector(CHAR_VAR,0,n-1);}
char *NewVector(char, long nl, long nh)
{
  char *v;
  v=(char *)malloc((size_t) ((nh-nl+1+LIB_VECTOR_END)*sizeof(char)));
  if (!v) LibError("allocation failure in Vectorc()");
  v+=-nl+LIB_VECTOR_END;
  for (long i=nl;i<=nh;i++) v[i]=CHAR_VAR;
  return v;
}
//-----------------------------------------------------------------------------
uchar *NewVector(uchar, long n) {  return NewVector(UCHAR_VAR, 0,n-1);}
uchar *NewVector(uchar, long nl, long nh)
{
  uchar *v;
  v=(uchar *)malloc((size_t) ((nh-nl+1+LIB_VECTOR_END)*sizeof(uchar)));
  if (!v) LibError("allocation failure in Vectorc()");
  v+=-nl+LIB_VECTOR_END;
  for (long i=nl;i<=nh;i++) v[i]=UCHAR_VAR;
  return v;
}
//=============================================================================
void FreeVector(float *v, long n){FreeVector(v,0,n-1);}
void FreeVector(float *v, long nl, long nh)
{
  free((LIB_FREE_ARG) (v+nl-LIB_VECTOR_END));
}
//-----------------------------------------------------------------------------
void FreeVector(double *v, long n){FreeVector(v,0,n-1);}
void FreeVector(double *v, long nl, long nh)
{
  free((LIB_FREE_ARG) (v+nl-LIB_VECTOR_END));
}
//-----------------------------------------------------------------------------
void FreeVector(int *v, long n){FreeVector(v,0,n-1);}
void FreeVector(int *v, long nl, long nh)
{
  free((LIB_FREE_ARG) (v+nl-LIB_VECTOR_END));
}
//-----------------------------------------------------------------------------
void FreeVector(char *v, long n){FreeVector(v,0,n-1);}
void FreeVector(char *v, long nl, long nh)
{
  free((LIB_FREE_ARG) (v+nl-LIB_VECTOR_END));
}
//-----------------------------------------------------------------------------
void FreeVector(uchar *v, long n){FreeVector(v,0,n-1);}
void FreeVector(uchar *v, long nl, long nh)
{
  free((LIB_FREE_ARG) (v+nl-LIB_VECTOR_END));
}
//=============================================================================
float VectorSum(float *v, long n){return VectorSum(v,0,n-1);}
float VectorSum(float *v, long nl, long nh)
{
  float s=0;
  for (int i=nl;i<=nh;i++)
    s+=v[i];
  return s;
}
//-----------------------------------------------------------------------------
double VectorSum(double *v, long n){return VectorSum(v,0,n-1);}
double VectorSum(double *v, long nl, long nh)
{
  double s=0;
  for (int i=nl;i<=nh;i++)
    s+=v[i];
	return s;
}
//=============================================================================
float VectorDist(float *v1, float *v2, long n){return VectorDist(v1,v2,0,n-1);}
float VectorDist(float *v1,float *v2, long nl,long nh)
{
  float d=0;
  for (int i=nl;i<=nh;i++)
    d+=(float)SQR(v1[i]-v2[i]);
  return (float)sqrt(d);
}
//-----------------------------------------------------------------------------
double VectorDist(double *v1, double *v2, long n){return VectorDist(v1,v2,0,n-1);}
double VectorDist(double *v1,double *v2, long nl,long nh)
{
  double d=0;
  for (int i=nl;i<=nh;i++)
    d+=SQR(v1[i]-v2[i]);
  return sqrt(d);
}
//========================================================================
// Calculate mean value of vector v
float VectorMean(float *v, long n) {return VectorMean(v,0,n-1);}
float VectorMean(float *v, long nl,long nh)
{
  float m=0;
  for (int i=nl;i<=nh;i++) m+=v[i];
  return (float)m/(nh-nl+1);
}
//------------------------------------------------------------------------
double VectorMean(double *v, long n) {return VectorMean(v,0,n-1);}
double VectorMean(double *v, long nl,long nh)
{
  double m=0;
  for (int i=nl;i<=nh;i++) m+=v[i];
  return (double)m/(nh-nl+1);
}
//========================================================================
// Calculate std deviation of values in vector v
float VectorStd(float *v, long n) {return VectorStd(v,0,n-1);}
float VectorStd(float *v, long nl,long nh)
{
  float m=VectorMean(v,nl,nh),d=0;
  for (int i=nl;i<=nh;i++) d+=(float)SQR(v[i]-m);
  if (nh!=nl) return (float)sqrt(d/(nh-nl));
  else        return (float)0;
}
//------------------------------------------------------------------------
double VectorStd(double *v, long n) {return VectorStd(v,0,n-1);}
double VectorStd(double *v, long nl,long nh)
{
  double m=VectorMean(v,nl,nh),d=0;
  for (int i=nl;i<=nh;i++) d+=(double)SQR(v[i]-m);
  if (nh!=nl) return (double)sqrt(d/(nh-nl));
  else        return (double)0;
}

//////////////////////////////// MATRIXES ////////////////////////////////
//========================================================================
float **NewMatrixF(long nrl,long nrh,long ncl,long nch){return NewMatrix(FLOAT_VAR,nrl,nrh,ncl,nch);}
float **NewMatrixF(long nr,long nc){return NewMatrix(FLOAT_VAR,0,nr-1,0,nc-1);}
float **NewMatrix(float,long nr,long nc){return NewMatrix(FLOAT_VAR,0,nr-1,0,nc-1);}
float **NewMatrix(float,long nrl,long nrh,long ncl,long nch)
{
  long i, nrow=nrh-nrl+1,ncol=nch-ncl+1;
  float **m;
  // allocate pointers to rows
  m=(float **) malloc((size_t)((nrow+LIB_VECTOR_END)*sizeof(float*)));
  if (!m) LibError("allocation failure 1 in matrix()");
  m += LIB_VECTOR_END;
  m -= nrl;
  // allocate rows and set pointers to them
  m[nrl]=(float *) malloc((size_t)((nrow*ncol+LIB_VECTOR_END)*sizeof(float)));
  if (!m[nrl]) LibError("allocation failure 2 in matrix()");
  m[nrl] += LIB_VECTOR_END;
  m[nrl] -= ncl;
  for(i=nrl+1;i<=nrh;i++) m[i]=m[i-1]+ncol;
  // Set array elements to zero
  for(long j=nrl;j<=nrh;j++)
    for(i=ncl;i<=nch;i++) m[j][i]=(float)0;
  // return pointer to array of pointers to rows
  return m;
}
//------------------------------------------------------------------------
double **NewMatrixD(long nrl,long nrh,long ncl,long nch){return NewMatrix(DOUBLE_VAR,nrl,nrh,ncl,nch);}
double **NewMatrixD(long nr,long nc){return NewMatrix(DOUBLE_VAR,0,nr-1,0,nc-1);}
double **NewMatrix(double,long nr,long nc){return NewMatrix(DOUBLE_VAR,0,nr-1,0,nc-1);}
double **NewMatrix(double,long nrl,long nrh,long ncl,long nch)
{
  long i, nrow=nrh-nrl+1,ncol=nch-ncl+1;
  double **m;
  // allocate pointers to rows
  m=(double **) malloc((size_t)((nrow+LIB_VECTOR_END)*sizeof(double*)));
  if (!m) LibError("allocation failure 1 in matrix()");
  m += LIB_VECTOR_END;
  m -= nrl;
  // allocate rows and set pointers to them
  m[nrl]=(double *) malloc((size_t)((nrow*ncol+LIB_VECTOR_END)*sizeof(double)));
  if (!m[nrl]) LibError("allocation failure 2 in matrix()");
  m[nrl] += LIB_VECTOR_END;
  m[nrl] -= ncl;
  for(i=nrl+1;i<=nrh;i++) m[i]=m[i-1]+ncol;
  // Set array elements to zero
  for(long j=nrl;j<=nrh;j++)
    for(i=ncl;i<=nch;i++) m[j][i]=(double)0;
  // return pointer to array of pointers to rows
  return m;
}
//------------------------------------------------------------------------
int **NewMatrixI(long nrl,long nrh,long ncl,long nch){return NewMatrix(INT_VAR,nrl,nrh,ncl,nch);}
int **NewMatrixI(long nr,long nc){return NewMatrix(INT_VAR,0,nr-1,0,nc-1);}
int **NewMatrix(int,long nr,long nc){return NewMatrix(INT_VAR,0,nr-1,0,nc-1);}
int **NewMatrix(int,long nrl,long nrh,long ncl,long nch)
{
  long i, nrow=nrh-nrl+1,ncol=nch-ncl+1;
  int **m;
  // allocate pointers to rows
  m=(int **) malloc((size_t)((nrow+LIB_VECTOR_END)*sizeof(int*)));
  if (!m) LibError("allocation failure 1 in matrix()");
  m += LIB_VECTOR_END;
  m -= nrl;
  // allocate rows and set pointers to them
  m[nrl]=(int *) malloc((size_t)((nrow*ncol+LIB_VECTOR_END)*sizeof(int)));
  if (!m[nrl]) LibError("allocation failure 2 in matrix()");
  m[nrl] += LIB_VECTOR_END;
  m[nrl] -= ncl;
  for(i=nrl+1;i<=nrh;i++) m[i]=m[i-1]+ncol;
  // Set array elements to zero
  for(long j=nrl;j<=nrh;j++)
    for(i=ncl;i<=nch;i++) m[j][i]=(int)0;
  // return pointer to array of pointers to rows
  return m;
}
//------------------------------------------------------------------------
char **NewMatrixC(long nrl,long nrh,long ncl,long nch){return NewMatrix(CHAR_VAR,nrl,nrh,ncl,nch);}
char **NewMatrixC(long nr,long nc){return NewMatrix(CHAR_VAR,0,nr-1,0,nc-1);}
char **NewMatrix(char,long nr,long nc){return NewMatrix(CHAR_VAR,0,nr-1,0,nc-1);}
char **NewMatrix(char,long nrl,long nrh,long ncl,long nch)
{
  long i, nrow=nrh-nrl+1,ncol=nch-ncl+1;
  char **m;
  // allocate pointers to rows
  m=(char **) malloc((size_t)((nrow+LIB_VECTOR_END)*sizeof(char*)));
  if (!m) LibError("allocation failure 1 in matrix()");
  m += LIB_VECTOR_END;
  m -= nrl;
  // allocate rows and set pointers to them
  m[nrl]=(char *) malloc((size_t)((nrow*ncol+LIB_VECTOR_END)*sizeof(char)));
  if (!m[nrl]) LibError("allocation failure 2 in matrix()");
  m[nrl] += LIB_VECTOR_END;
  m[nrl] -= ncl;
  for(i=nrl+1;i<=nrh;i++) m[i]=m[i-1]+ncol;
  // Set array elements to zero
  for(long j=nrl;j<=nrh;j++)
    for(i=ncl;i<=nch;i++) m[j][i]=(char)0;
  // return pointer to array of pointers to rows
  return m;
}
//------------------------------------------------------------------------
uchar **NewMatrix(uchar, long nr, long nc){return NewMatrix(UCHAR_VAR,0,nr-1,0,nc-1);}
uchar **NewMatrix(uchar, long nrl, long nrh, long ncl, long nch)
{
  long i, nrow=nrh-nrl+1,ncol=nch-ncl+1;
  uchar **m;
  // allocate pointers to rows
  m=(uchar **) malloc((size_t)((nrow+LIB_VECTOR_END)*sizeof(uchar*)));
  if (!m) LibError("allocation failure 1 in matrix()");
  m += LIB_VECTOR_END;
  m -= nrl;
  // allocate rows and set pointers to them
  m[nrl]=(uchar *) malloc((size_t)((nrow*ncol+LIB_VECTOR_END)*sizeof(uchar)));
  if (!m[nrl]) LibError("allocation failure 2 in matrix()");
  m[nrl] += LIB_VECTOR_END;
  m[nrl] -= ncl;
  for(i=nrl+1;i<=nrh;i++) m[i]=m[i-1]+ncol;
  // Set array elements to zero
  for(long j=nrl;j<=nrh;j++)
    for(i=ncl;i<=nch;i++) m[j][i]=(uchar)0;
  // return pointer to array of pointers to rows
  return m;
}
//========================================================================
void FreeMatrix(float **m, long nr, long nc){FreeMatrix(m,0,nr-1,0,nc-1);}
void FreeMatrix(float **m, long nrl, long nrh, long ncl, long nch)
{
  free((LIB_FREE_ARG) (m[nrl]+ncl-LIB_VECTOR_END));
  free((LIB_FREE_ARG) (m+nrl-LIB_VECTOR_END));
}
//------------------------------------------------------------------------
void FreeMatrix(double **m, long nr, long nc){FreeMatrix(m,0,nr-1,0,nc-1);}
void FreeMatrix(double **m, long nrl, long nrh, long ncl, long nch)
{
  free((LIB_FREE_ARG) (m[nrl]+ncl-LIB_VECTOR_END));
  free((LIB_FREE_ARG) (m+nrl-LIB_VECTOR_END));
}
//------------------------------------------------------------------------
void FreeMatrix(int **m, long nr, long nc){ FreeMatrix(m,0,nr-1,0,nc-1);}
void FreeMatrix(int **m, long nrl, long nrh, long ncl, long nch)
{
  free((LIB_FREE_ARG) (m[nrl]+ncl-LIB_VECTOR_END));
  free((LIB_FREE_ARG) (m+nrl-LIB_VECTOR_END));
}
//------------------------------------------------------------------------
void FreeMatrix(char **m, long nr, long nc){FreeMatrix(m,0,nr-1,0,nc-1);}
void FreeMatrix(char **m, long nrl, long nrh, long ncl, long nch)
{
  free((LIB_FREE_ARG) (m[nrl]+ncl-LIB_VECTOR_END));
  free((LIB_FREE_ARG) (m+nrl-LIB_VECTOR_END));
}
//------------------------------------------------------------------------
void FreeMatrix(uchar **m, long nr, long nc){FreeMatrix(m,0,nr-1,0,nc-1);}
void FreeMatrix(uchar **m, long nrl, long nrh, long ncl, long nch)
{
  free((LIB_FREE_ARG) (m[nrl]+ncl-LIB_VECTOR_END));
  free((LIB_FREE_ARG) (m+nrl-LIB_VECTOR_END));
}
//------------------------------------------------------------------------


#ifdef LIB_VECTOR_END
  #undef LIB_VECTOR_END
  #undef LIB_FREE_ARG
#endif









