//*****************************************************************************
//* Library of different utilities. Headers file.
//*    Some routines were taken from "Numerical Recipes" by Press, 
//*                                              Teukolsky, Vetterling, Flanery
//*    Realized by P.V.Nazarov, 2002
//*****************************************************************************
#ifndef LIBUTILS_H_20020702
#define LIBUTILS_H_20020702
//=============================================================================
#include <stdio.h>
#include <conio.h>
#include <stddef.h>
#include <stdlib.h>
#include <math.h>
#pragma hdrstop
//=============================================================================
typedef unsigned char uchar;
typedef unsigned long ulong;
// Definition of typizied constants
#define FLOAT_VAR   (float)0
#define DOUBLE_VAR  (double)0
#define INT_VAR     (int)0
#define CHAR_VAR    (char)0
#define UCHAR_VAR   (unsigned char)0
// Mathematical constants
#define M_EXP       2.71828182845904523536
#define M_LOG2E     1.44269504088896340736
#define M_LOG10E    0.434294481903251827651
#define M_LN2       0.693147180559945309417
#define M_LN10      2.30258509299404568402
#define M_PI        3.14159265358979323846
#define M_PI_2      1.57079632679489661923
#define M_PI_4      0.785398163397448309616
#define M_1_PI      0.318309886183790671538
#define M_2_PI      0.636619772367581343076
#define M_1_SQRTPI  0.564189583547756286948
#define M_2_SQRTPI  1.12837916709551257390
#define M_SQRT2     1.41421356237309504880
#define M_SQRT_2    0.707106781186547524401

#define M_SQRT3     1.73205080756887729352
#define M_SQRT3_2   0.866025403784438646764
#define M_SQRT3_4   0.433012701892219323382

// internal definitions
#define LIB_VECTOR_END  1
#define LIB_FREE_ARG  char*
//=============================================================================
static double radarg;
#define GR2RAD(a) ((radarg=(a)) == 0.0 ? 0.0 : radarg*M_PI/180)
static double grarg;
#define RAD2GR(a) ((radarg=(a)) == 0.0 ? 0.0 : 180*radarg/M_PI)

static double powarg1,powarg2;
#define POW(a,b) ((powarg1=(a),powarg2=(b)) == 0.0 ? 0.0 : exp(powarg2*log(powarg1)))

static double sqrarg;
#define SQR(a) ((sqrarg=(a)) == 0.0 ? 0.0 : sqrarg*sqrarg)

static double dmaxarg1,dmaxarg2;
#define DMAX(a,b) (dmaxarg1=(a),dmaxarg2=(b),(dmaxarg1) > (dmaxarg2) ?\
(dmaxarg1) : (dmaxarg2))

static double dminarg1,dminarg2;
#define DMIN(a,b) (dminarg1=(a),dminarg2=(b),(dminarg1) < (dminarg2) ?\
(dminarg1) : (dminarg2))

static float maxarg1,maxarg2;
#define FMAX(a,b) (maxarg1=(a),maxarg2=(b),(maxarg1) > (maxarg2) ?\
(maxarg1) : (maxarg2))

static float minarg1,minarg2;
#define FMIN(a,b) (minarg1=(a),minarg2=(b),(minarg1) < (minarg2) ?\
(minarg1) : (minarg2))

static long lmaxarg1,lmaxarg2;
#define LMAX(a,b) (lmaxarg1=(a),lmaxarg2=(b),(lmaxarg1) > (lmaxarg2) ?\
(lmaxarg1) : (lmaxarg2))

static long lminarg1,lminarg2;
#define LMIN(a,b) (lminarg1=(a),lminarg2=(b),(lminarg1) < (lminarg2) ?\
(lminarg1) : (lminarg2))

static int imaxarg1,imaxarg2;
#define IMAX(a,b) (imaxarg1=(a),imaxarg2=(b),(imaxarg1) > (imaxarg2) ?\
(imaxarg1) : (imaxarg2))

static int iminarg1,iminarg2;
#define IMIN(a,b) (iminarg1=(a),iminarg2=(b),(iminarg1) < (iminarg2) ?\
(iminarg1) : (iminarg2))

#define SIGN(a,b) ((b) >= 0.0 ? fabs(a) : -fabs(a))

//-----------------------------------------------------------------------------
// Errors processing (for console applications only)
void LibError(char error_text[]);
void LibError(char error_text[],int exit_code);
void LibError(char error_text[],char error_proc[],int exit_code);
//-----------------------------------------------------------------------------
// round to nearest
//double pow(double r,double n);
double pow(double x, int n);

//-----------------------------------------------------------------------------
// round to nearest
int round(double r);
int Round(double r);
//-----------------------------------------------------------------------------
char UpCase(char c);
char upcase(char c);
//-----------------------------------------------------------------------------
char DownCase(char c);
char downcase(char c);

/////////////////////////////// VECTORS ///////////////////////////////////////
//-----------------------------------------------------------------------------
// Listed below functions allocate memory for a vector

// allocate a float vector with subscript range v[nl..nh]
float *NewVector(float, long nl, long nh);
float *NewVectorF(long nl, long nh);
// allocate a float vector with subscript range v[0..n-1]
float *NewVector(float, long n);
float *NewVectorF(long n);
// allocate a double vector with subscript range v[nl..nh]
double *NewVector(double, long nl, long nh);
double *NewVectorD(long nl, long nh);
// allocate a double vector with subscript range v[0..n-1]
double *NewVector(double, long n);
double *NewVectorD(long n);
// allocate an integer vector with subscript range v[nl..nh]
int *NewVector(int, long nl, long nh);
int *NewVectorI(long nl, long nh);
// allocate an integer vector with subscript range v[0..n-1]
int *NewVector(int, long n);
int *NewVectorI(long n);
// allocate a char vector with subscript range v[nl..nh]
char *NewVector(char, long nl, long nh);
char *NewVectorC(long nl, long nh);
// allocate a char vector with subscript range v[0..n-1]
char *NewVector(char, long n);
char *NewVectorC(long n);
// allocate a char vector with subscript range v[nl..nh]
uchar *NewVector(uchar, long nl, long nh);
// allocate a char vector with subscript range v[0..n-1]
uchar *NewVector(uchar, long n);

//-----------------------------------------------------------------------------
// Listed below functions aimed to free memory
// allocated by VectorX functions

void FreeVector(float *v, long nl, long nh);
void FreeVector(float *v, long n);

void FreeVector(double *v, long nl, long nh);
void FreeVector(double *v, long n);

void FreeVector(int *v, long nl, long nh);
void FreeVector(int *v, long n);

void FreeVector(char *v, long nl, long nh);
void FreeVector(char *v, long n);

void FreeVector(uchar *v, long nl, long nh);
void FreeVector(uchar *v, long n);
//-----------------------------------------------------------------------------
// Summarize all elements of vector v
float VectorSum(float *v, long n);
float VectorSum(float *v, long nl,long nh);
double VectorSum(double *v, long n);
double VectorSum(double *v, long nl,long nh);

// Calculate multidim distance between vectors v1,v2
//            d = sqrt( sum((v1-v2)^2) )
float VectorDist(float *v1,float *v2, long n);
float VectorDist(float *v1,float *v2, long nl,long nh);
double VectorDist(double *v1,double *v2, long n);
double VectorDist(double *v1,double *v2, long nl,long nh);

// Calculate mean value of vector v
float VectorMean(float *v, long n);
float VectorMean(float *v, long nl,long nh);
double VectorMean(double *v, long n);
double VectorMean(double *v, long nl,long nh);

// Calculate std deviation of values in vector v
float VectorStd(float *v, long n);
float VectorStd(float *v, long nl,long nh);
double VectorStd(double *v, long n);
double VectorStd(double *v, long nl,long nh);

/////////////////////////////// MATRIXES //////////////////////////////////////
//-----------------------------------------------------------------------------
float **NewMatrix(float, long nrl, long nrh, long ncl, long nch);
float **NewMatrixF(long nrl, long nrh, long ncl, long nch);
// Allocates a float matrix with subscript range m[nrl..nrh][ncl..nch]
float **NewMatrix(float, long nr, long nc);
float **NewMatrixF(long nr, long nc);
// Allocates a float matrix with subscript range m[0..nr-1][0..nr-1]

double **NewMatrix(double, long nrl, long nrh, long ncl, long nch);
double **NewMatrixD(long nrl, long nrh, long ncl, long nch);
// Allocates a double matrix with subscript range m[nrl..nrh][ncl..nch]
double **NewMatrix(double, long nr, long nc);
double **NewMatrixD(long nr, long nc);
// Allocates a double matrix with subscript range m[0..nr-1][0..nr-1]

int **NewMatrix(int, long nrl, long nrh, long ncl, long nch);
int **NewMatrixI(long nrl, long nrh, long ncl, long nch);
// Allocates an int matrix with subscript range m[nrl..nrh][ncl..nch]
int **NewMatrix(int, long nr, long nc);
int **NewMatrixI(long nr, long nc);
// Allocates an int matrix with subscript range m[0..nr-1][0..nr-1]

char **NewMatrix(char, long nrl, long nrh, long ncl, long nch);
char **NewMatrixC(long nrl, long nrh, long ncl, long nch);
// Allocates a char matrix with subscript range m[nrl..nrh][ncl..nch]
char **NewMatrix(char, long nr, long nc);
char **NewMatrixC(long nr, long nc);
// Allocates a char matrix with subscript range m[0..nr-1][0..nr-1]
uchar **NewMatrix(uchar, long nrl, long nrh, long ncl, long nch);
// Allocates a char matrix with subscript range m[nrl..nrh][ncl..nch]
uchar **NewMatrix(uchar, long nr, long nc);
// Allocates a char matrix with subscript range m[0..nr-1][0..nr-1]
//-----------------------------------------------------------------------------
void FreeMatrix(float **m, long nrl, long nrh, long ncl, long nch);
void FreeMatrix(float **m, long nr, long nc);

void FreeMatrix(double **m, long nrl, long nrh, long ncl, long nch);
void FreeMatrix(double **m, long nr, long nc);

void FreeMatrix(int **m, long nrl, long nrh, long ncl, long nch);
void FreeMatrix(int **m, long nr, long nc);

void FreeMatrix(char **m, long nrl, long nrh, long ncl, long nch);
void FreeMatrix(char **m, long nr, long nc);
void FreeMatrix(uchar **m, long nrl, long nrh, long ncl, long nch);
void FreeMatrix(uchar **m, long nr, long nc);

//-----------------------------------------------------------------------------



//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
#endif
